@@ -17,9 +17,10 @@ from magnetodb.api.amz.dynamodb.action import DynamoDBAction
 from magnetodb.api.amz.dynamodb import parser
 
 from magnetodb import storage
-from magnetodb.storage import models
+from magnetodb.api.amz.dynamodb.exception import AWSValidationException
+from magnetodb.api.amz.dynamodb.exception import AWSErrorResponseException
 
-from magnetodb.common import exception
+from magnetodb.storage import models
 
 
 class PutItemDynamoDBAction(DynamoDBAction):
@@ -46,7 +47,7 @@ class PutItemDynamoDBAction(DynamoDBAction):
                                 "required": [parser.Props.VALUE],
                                 "properties": {
                                     parser.Props.VALUE:
-                                        parser.Types.ITEM_VALUE,
+                                        parser.Types.TYPED_ATTRIBUTE_VALUE,
                                 }
                             },
                         ]
@@ -54,12 +55,7 @@ class PutItemDynamoDBAction(DynamoDBAction):
                 }
             },
 
-            parser.Props.ITEM: {
-                "type": "object",
-                "patternProperties": {
-                    parser.ATTRIBUTE_NAME_PATTERN: parser.Types.ITEM_VALUE
-                }
-            },
+            parser.Props.ITEM: parser.Types.ITEM,
 
             parser.Props.RETURN_CONSUMED_CAPACITY: (
                 parser.Types.RETURN_CONSUMED_CAPACITY
@@ -93,10 +89,12 @@ class PutItemDynamoDBAction(DynamoDBAction):
             )
 
             # parse return_values param
-            return_values = self.action_params.get(
+            return_values_json = self.action_params.get(
                 parser.Props.RETURN_VALUES, parser.Values.RETURN_VALUES_NONE
             )
 
+            return_values = models.InsertReturnValuesType(return_values_json)
+
             # parse return_item_collection_metrics
             return_item_collection_metrics = self.action_params.get(
                 parser.Props.RETURN_ITEM_COLLECTION_METRICS,
@@ -108,23 +106,26 @@ class PutItemDynamoDBAction(DynamoDBAction):
                 parser.Values.RETURN_CONSUMED_CAPACITY_NONE
             )
         except Exception:
-            raise exception.ValidationException()
+            raise AWSValidationException()
 
         try:
             # put item
-            result = storage.put_item(
+            result, old_item = storage.put_item(
                 self.context,
-                models.PutItemRequest(table_name, item_attributes),
+                table_name,
+                item_attributes,
+                return_values,
                 if_not_exist=False,
-                expected_condition_map=expected_item_conditions)
+                expected_condition_map=expected_item_conditions
+            )
 
             if not result:
-                raise exception.AWSErrorResponseException()
+                raise AWSErrorResponseException()
 
             # format response
             response = {}
 
-            if return_values != parser.Values.RETURN_VALUES_NONE:
+            if old_item:
                 response[parser.Props.ATTRIBUTES] = (
                     parser.Parser.format_item_attributes(item_attributes)
                 )
@@ -134,8 +135,7 @@ class PutItemDynamoDBAction(DynamoDBAction):
                 response[parser.Props.ITEM_COLLECTION_METRICS] = {
                     parser.Props.ITEM_COLLECTION_KEY: {
                         parser.Parser.format_item_attributes(
-                            models.AttributeValue(models.ATTRIBUTE_TYPE_STRING,
-                                                  "key")
+                            models.AttributeValue("S", "key")
                         )
                     },
                     parser.Props.SIZE_ESTIMATED_RANGE_GB: [0]
@@ -150,7 +150,7 @@ class PutItemDynamoDBAction(DynamoDBAction):
                 )
 
             return response
-        except exception.AWSErrorResponseException as e:
+        except AWSErrorResponseException as e:
             raise e
         except Exception:
-            raise exception.AWSErrorResponseException()
+            raise AWSErrorResponseException()
